' DelayedAutoStart v1.0

'  This script starts programs, files, documents, shortcuts etc. delayed.
' The delayed files must lie in a folder named "DelayedAutoStart"
' that must be in the parent directory of the directory this
' script file is lying in.
' You need to COPY the script into your "Autostart" or "Startup"
' folder in your Start-Programs menu (do really copy it, not make
' just a shortcut to the script) and you need to put the
' "DelayedAutoStart" folder in your Start-Programs menu, too.

'  In the "DelayedAutoStart" folder, you will normally place shortcuts
' like in the "Startup" folder itself.
' If you do this, the shortcuts will be started delayed by 30 seconds.
'  The order is by hazard and depends on the order stored on the disk.
' But if you put something like [80sec] or [20.5sec] etc. into the name
' of a shortcut, then the script will delay the execution by the specified
' amount of time (in the samples 80 seconds or 20.5 seconds). The value
' is a floating point value in seconds from the beginning of starting
' this script file.
' The dot of the floating point number can be both a "." or a ",".
' Both are supported and interpreted as a floating point dot.

'  Sample configuration cutout:
' Notice that for a shortcut, the extension is ".lnk" but you usually don't see it.
' Imagine the contents of the "Startup" folder:
'   C:\Documents and Settings\ThatsMeUser\Startmenu\Programs\Startup\notepad.exe.lnk
'   C:\Documents and Settings\ThatsMeUser\Startmenu\Programs\Startup\DelayedAutoStart.vbs
' Notice that the "DelayedAutoStart.vbs" script is *copied* into the Startup folder
' so not only as shortcut.
' Now imagine the contents of the "DelayedAutoStart" folder:
'   C:\Documents and Settings\ThatsMeUser\Startmenu\Programs\DelayedAutoStart\iexplore.lnk
'   C:\Documents and Settings\ThatsMeUser\Startmenu\Programs\DelayedAutoStart\iexplore[32,5sec].lnk
'   C:\Documents and Settings\ThatsMeUser\Startmenu\Programs\DelayedAutoStart\TheClock[20sec].lnk
' Imagine each of the ".lnk" shortcuts to be shortcuts to the according executable files.
' What happens when the user "ThatsMeUser" logs in:
'   notepad.exe ist started immediately, because it is in the "Startup" folder.
'   as well the "DelayedAutoStart.vbs" script will be started.
'   The script reads new the "DelayedAutoStart" folder and starts the following files
'   with the following delays in the following order:
'   TheClock.exe will be started with 20 seconds delay.
'   iexplore.exe will be started with 30 seconds delay (30 seconds is the default if not sepcified in the filename).
'   Another iexplore.exe will be started with 32.5 seconds delay.

'  Requirements:
' Internet Explorer 5.0 or higher must have been installed.

'  Script engine version:
' Since RegExp (regular expression) object is used in this script, you
' need VB Script Edition version 5.0 or higher.
' That is, you should have installed Internet Explorer 5.0 or higher.

' Copyright (c) 2005-2006, Roman Komary

'  This software is freeware. You may copy and redistribute this script
' as long as all copyright notices remain intact and the files and
' product materials delivered with this software package are not sold,
' disassembled, reverse-engineered or modified in any way without
' the author's explicit written permission.
'  In respect to this, you are allowed to take all or parts of this script
' code and integrate it, unmodified or modified, into your own code segments.

'  The author of this script accepts no responsibility for damages
' or other malfunctions that are caused directly or indirectly by
' the use of this script and makes NO WARRANTIES or representation,
' either express or implied, with respect to this software.
' This software is provided "AS IS", and you, its user, assume
' the entire risk when you use it.




'-------------------------------------------------------------------------
'  : szDir = a full path. *must not* end with a \ or
'            else it will not be changed
'  > a string with the parent directory without trailing \ char.
'    If the function did not do anything, this will be
'    the same string as 'szDir'.
'    IMPORTANT Sample: "C:\a_path" will result in "C:".
'     In this case you will have to add the \ char by hand.
Function GetParentDir( szDir )
    GetParentDir = szDir
    i = Len(szDir)
    continue = true
    while( continue )
        if( i >= 1 ) then
            ch = Mid(szDir,i,1)
            if( ch = "\" ) then
                continue = false
                GetParentDir = Left( szDir, i-1 )
            else
                GetParentDir = Left( szDir, i )
            end if
            i = i - 1
        else
            continue = false
        end if
    wend
End Function

'-------------------------------------------------------------------------
' Returns the Current Working Directory.
' This is *not* the windows-current-directory (or work directory if started from a shortcut).
' It is the directory of this script file executed.
Function GetWorkDir()
    szScriptFile = WScript.ScriptFullName
    szScriptDir = GetParentDir( szScriptFile )
    GetWorkDir = szScriptDir
End Function

'-------------------------------------------------------------------------
' Starts a program, file, document, shortcut.
Sub StartFile( sFile )
    Set wsh = CreateObject("WScript.Shell")

    If( LCase(Right(sFile, 4)) = ".lnk" ) Then
        ' Here we make a special handling for shortcuts.
        ' If you pass a .lnk file directly to wsh.Run(), then some other application
        ' is started or whatever (with me it's 7-zip that is started and the folder
        ' of the target file is passed to it).
        ' So I have to read out the contents of the shortcut and start the target
        ' file explicitely.

        Set lnk = wsh.CreateShortcut( sFile )

        nWndStyle = lnk.WindowStyle     ' 1, 3 or 7 could occur
        bWaitOnReturn = false

        Call wsh.Run( """" & lnk.TargetPath & """", nWndStyle, bWaitOnReturn )
        ' Note: Unfortunately the lnk.WorkingDirectory property cannot be passed to
        ' wsh.Run() and I did not find another way to execute a shortcut.
    Else
        nWndStyle = 1   ' 0 = hide app window
                        ' 1 = display app window
                        ' 2 = open app window minimized and activate it
                        ' 3 = Activates the window and displays it as a maximized window
                        ' 7 = open app window minimized without activating it
                        ' 8 = display app window without activating it
        bWaitOnReturn = false

        Call wsh.Run( """" & sFile & """", nWndStyle, bWaitOnReturn )
    End If
End Sub

'-------------------------------------------------------------------------
' Sorts the first array in ascending order and also sorts the
' second array in the order of the first array.
Sub SortAr( ar1, ar2, cnt )
    For m = 1 to cnt - 1
        For n = 2 to cnt
            If( ar1(m) > ar1(n) ) Then
                Dim hlp1, hlp2

                hlp1   = ar1(m)
                ar1(m) = ar1(n)
                ar1(n) = hlp1

                hlp2   = ar2(m)
                ar2(m) = ar2(n)
                ar2(n) = hlp2
            End If
        Next
    Next
End Sub

'-------------------------------------------------------------------------




Dim originalLoc
originalLoc = SetLocale( "en-us" )    ' so that floating point dot is a "." and never a ","


fDefault_Delay = 30.0   ' floating point value in seconds


tStart = Now

Dim wsh, fso
Set wsh = WScript.CreateObject( "WScript.Shell" )
Set fso = CreateObject("Scripting.FileSystemObject")

sWorkDir    = GetWorkDir
sDelayedDir = GetParentDir( sWorkDir ) + "\DelayedAutoStart"
If( not fso.FolderExists( sDelayedDir ) ) Then
    Call wsh.Popup( "No DelayedAutoStartup folder. Nothing to start delayed." & Chr(10) & "This dialog will close in 3 seconds.", 3 )
    WScript.Quit
End If

' First we scan the folder for all files to start and query
' its delay time value.
cFiles = 0

Set oDelayedFolder  = fso.GetFolder( sDelayedDir )
Set oFiles          = oDelayedFolder.Files
Set oRegExp         = new RegExp
oRegExp.Pattern     = "\[([0-9.,]+)sec\]"
oRegExp.IgnoreCase  = false
oRegExp.Global      = false     ' match only the first pattern instead of all found
For each oFile in oFiles
    If( LCase(oFile.Name) <> "delayedautostart.vbs" ) Then
        cFiles = cFiles + 1
        ReDim Preserve theFiles( cFiles )
        ReDim Preserve theDelays( cFiles )

        theFiles(cFiles) = oFile.Path

        theDelays(cFiles) = CDbl( fDefault_Delay )
        Set oMatches = oRegExp.Execute( oFile.Name )
        If( oMatches.Count > 0 ) Then
            theDelays(cFiles) = CDbl( Replace( oMatches(0).SubMatches(0), ",", "." ) )
        End If
    End If
Next

' Sort in ascending delay time order.
Call SortAr( theDelays, theFiles, cFiles )

' Now start each of the files and await their delays.
'Call wsh.Popup( "Starting DelayedAutoStartup folder now." & Chr(10) & "This dialog will close in 3 seconds.", 3 )

For i = 1 to cFiles
    tOfs = CDbl( Round( CDbl(Now - tStart) * 24.0 * 60.0 * 60.0 ) )
    ' Note: Unfortunately, the Date type (and so functions like Now) have only
    ' a resolution of a second. Not below. So the delays are not exact if
    ' you specify delay values with floating point numbers (so below a second).
    ' But it will at least retain the order of starting the files.
    If( theDelays(i) > tOfs ) Then WScript.Sleep( (theDelays(i) - tOfs) * 1000 )

    Call StartFile( theFiles(i) )
Next


SetLocale( originalLoc )
